/************************************
*    SRF05 Example for PIC18F4410   *
* Using single trigger and echo pin *
*      By James Henderson, 2008     *
************************************/

#include <p18cxxx.h>
#include "stdio.h"
#include "stdlib.h"
#include "SRF05.h"

// Configuration Bit Values
/** CONFIGURATION **************************************************/
        #pragma config OSC      = INTIO67
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = ON
        #pragma config BOREN    = OFF
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = OFF
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
        #pragma config STVREN   = ON
        #pragma config LVP      = OFF
        #pragma config XINST    = OFF       			// Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
        #pragma config CPB      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
        #pragma config WRTB     = OFF       			// Boot Block Write Protection
        #pragma config WRTC     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
        #pragma config EBTRB    = OFF

unsigned char s[21]; 						// used to print data to the LCD03

void main(void)
{
unsigned int range;

	setup();
	setup_lcd();							// Clear LCD03 screen and hide cursor
	set_cursor(1);
	sprintf(s, "SRF05 Example");
	print_buff();

	while(1)
	{
		SetTimer(6500);						// Start a 50mS timer running with timer 0

		TRISDbits.TRISD3 = 0;				// Set the pin for SRF05 to output			
		send_pulse();						// Send a 10uS pulse to trigger the SRF05			
		TRISDbits.TRISD3 = 1;
		while(!ECHO);						// Wait for echo line to go high
		measure_pulse();					// Measure the length of the incoming pulse using timer 1

		range = (TMR1H << 8) + TMR1L;		// Get timer 1 values high byte and low byte into range
		range /= 58;						// Range is a reading in uS dividing this by 58 will give us the range
		sprintf(s, "Range: %i  ", range);
		set_cursor(21);
		print_buff();

		Wait4Timer();						// Wait for the 50mS timer to finish before triggering the SRF05 again
	}
}

/*****************
* SRF05 routines *
*****************/

void send_pulse(void)
{
	TMR1H = 255;					// Set the clock to only need 2 counts to roll over				
	TMR1L = 253;					// This will give us 10uS before it rolls over

	TRIG = 1; 						// Pin high to trigger ping from SRF05;

	PIR1bits.TMR1IF = 0;			// Clear timer 1 interupt flag
	T1CONbits.TMR1ON = 1;			// Start timer 1 running for 10us
	while(PIR1bits.TMR1IF == 0);	// Wait for interupt to be set
	T1CONbits.TMR1ON = 0;
	TRIG = 0; 						// Pin output for SRF05 low
	TMR1H = 0;						// Clear timer
	TMR1L = 0;					
}

void measure_pulse(void)
{
	T1CONbits.TMR1ON = 1;			// Start timer 1 counting
	while(ECHO);					// Wait for echo line to go low
	T1CONbits.TMR1ON = 0;
}

/*****************
* Timer Routines *
*****************/
void SetTimer(unsigned int time)
{
	TMR0H = 0-(time/256);
	TMR0L = 0-(time&0xff);
	INTCONbits.TMR0IF = 0;
}

void Wait4Timer(void)
{
	while(INTCONbits.TMR0IF==0);
}

/*************
* Chip setup *
*************/

void setup(void)
{	
	OSCCON = 0x70;				// set 8MHz

	TRISDbits.TRISD3 = 0;		// pin output for SRF05 
	LATDbits.LATD3 = 0;			

	T0CON = 0x83;				// 16bit, 64:1 prescaler
	T1CON = 0x10;				// Timer 1 16bit: 2:1 prescaler

	ADCON1 = 0x0f;				// all ports to digital
	TRISC = 0x1F;
	SSPSTAT = 0x80;				// slew diabled
	SSPCON1 = 0x38;				// master mode enabled
	SSPADD = 0x11;				// 100khz
}

/**********************
* Functions for LCD03 *
**********************/

void setup_lcd(void)
{
	SSPCON2bits.SEN = 1;	// innitiate a start bit
	while(SSPCON2bits.SEN);	// Wait for start bit to end
	PIR1bits.SSPIF = 0;		// Clear the i2c interupt flag
	SSPBUF = 0xC6;			// Address of LCD03
	while(!PIR1bits.SSPIF);	// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x00;			// Command register of LCD03
	while(!PIR1bits.SSPIF);		
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x04;			// command to hide cursor
	while(!PIR1bits.SSPIF);	
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x0C;			// command to clear screen
	while(!PIR1bits.SSPIF);	
	SSPCON2bits.PEN = 1;	// Send stop bit
	while(SSPCON2bits.PEN);	// Wait for stop bit to finish
}

void print_buff(void)
{
unsigned char index = 0;
	SSPCON2bits.SEN = 1;				// innitiate a start bit
	while(SSPCON2bits.SEN);				// Wait for start bit to end
	PIR1bits.SSPIF = 0;					// Clear the i2c interupt flag
	SSPBUF = 0xC6;						// Address of LCD03
	while(!PIR1bits.SSPIF);				// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x00;						// Command register of LCD03
	while(!PIR1bits.SSPIF);	
	while(s[index])						// Send ascii string to LCD03 untill a null reached
	{
		PIR1bits.SSPIF = 0;		
		SSPBUF = s[index++];			// Command register of LCD03
		while(!PIR1bits.SSPIF);	
	}
	SSPCON2bits.PEN = 1;				// Send stop bit
	while(SSPCON2bits.PEN);
}	

void set_cursor(unsigned char pos)
{
	SSPCON2bits.SEN = 1;	// innitiate a start bit
	while(SSPCON2bits.SEN);	// Wait for start bit to end
	PIR1bits.SSPIF = 0;		// Clear the i2c interupt flag
	SSPBUF = 0xC6;			// Address of LCD03
	while(!PIR1bits.SSPIF);	// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x00;			// Command register of LCD03
	while(!PIR1bits.SSPIF);	// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = 0x02;			// Command to set cursor possition
	while(!PIR1bits.SSPIF);	// Wait for interupt flag to be set indicating and of a transmission
	PIR1bits.SSPIF = 0;		
	SSPBUF = pos;			// where to set cursor to
	while(!PIR1bits.SSPIF);

	SSPCON2bits.PEN = 1;	// Send stop bit
	while(SSPCON2bits.PEN);	// Wait for stop bit to finish
}